@@ -22,7 +22,7 @@
 import os, sys
 
 
-VERSION = "0.35.10"
+VERSION = "0.36.3"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
 if sys.platform.startswith("win"):
@@ -113,6 +113,7 @@ TAG_PROP_SUB = "sub"
 TAG_PROP_LEFT = "left"
 TAG_PROP_CENTER = "center"
 TAG_PROP_RIGHT = "right"
+TAG_PROP_FILL = "fill"
 TAG_PROPERTIES = [TAG_WEIGHT, TAG_FOREGROUND, TAG_BACKGROUND, TAG_STYLE, TAG_UNDERLINE,
                   TAG_STRIKETHROUGH, TAG_SCALE, TAG_FAMILY, TAG_JUSTIFICATION, TAG_LINK]
 TAG_SEPARATOR = "separator"
@@ -144,10 +145,11 @@ CHAR_NEWLINE = "\n"
 CHAR_NEWPAGE = "\x0c"
 CHAR_CR = "\r"
 CHAR_TAB = "\t"
-CHAR_LISTBUL = "•"
+CHARS_LISTBUL = ["•", "◇", "▪", "-", "→", "⇒"]
+CHARS_LISTNUM = [".", ")", "-", ">"]
+NUM_CHARS_LISTBUL = len(CHARS_LISTBUL)
+NUM_CHARS_LISTNUM = len(CHARS_LISTNUM)
 CHAR_LISTARR = "▸"
-CHAR_LISTBUL2 = "◇"
-CHAR_LISTBUL3 = "▪"
 CHAR_LISTTODO = "☐"
 CHAR_LISTDONEOK= "☑"
 CHAR_LISTDONEFAIL= "☒"
@@ -193,6 +195,7 @@ STR_IGNORE = "ignore"
 STR_KEY_RETURN = "Return"
 STR_KEY_DELETE = "Delete"
 STR_KEY_TAB = "Tab"
+STR_KEY_SHIFT_TAB = "ISO_Left_Tab"
 STR_KEY_SPACE = "space"
 STR_KEY_MENU = "Menu"
 STR_KEY_UP = "Up"
@@ -230,9 +233,9 @@ STOCKS_N_FILES = [
 'object-rotate-left.png', 'object-rotate-right.png',
 'to_pdf.png', 'to_txt.png', 'to_html.png', 'to_cherrytree.png', 'export_from_cherrytree.png', STR_STOCK_CT_IMP + '.png',
 'from_cherrytree.png', 'from_txt.png', 'from_html.png', 'cherrytree.png', 'quit-app.png',
-'new-instance.png', 'toolbar.png', 'cherries.png', 'tree-node-add.png',
+'new-instance.png', 'toolbar.png', 'cherries.png', 'tree-node-dupl.png', 'tree-node-add.png',
 'tree-subnode-add.png', 'help-contents.png', 'index.png', 'timestamp.png',
-'calendar.png', 'horizontal_rule.png', 'file_icon.png']
+'calendar.png', 'horizontal_rule.png', 'file_icon.png', 'pin.png', 'pin-add.png', 'pin-remove.png']
 
 NODES_STOCKS = ['node_bullet', 'node_no_icon', 'cherry_black',
                 'cherry_blue', 'cherry_cyan', 'cherry_green',
@@ -339,6 +342,8 @@ UI_INFO = """
             <separator/>
             <menuitem action='EnaDisSpellCheck'/>
             <separator/>
+            <menuitem action='CutPlainText'/>
+            <menuitem action='CopyPlainText'/>
             <menuitem action='PastePlainText'/>
             <separator/>
             <menuitem action='CutRow'/>
@@ -374,6 +379,7 @@ UI_INFO = """
             <menuitem action='JustifyLeft'/>
             <menuitem action='JustifyCenter'/>
             <menuitem action='JustifyRight'/>
+            <menuitem action='JustifyFill'/>
         </menu>
 
         <menu action='TreeMenu'>
@@ -386,9 +392,7 @@ UI_INFO = """
             <menuitem action='FindNode'/>
             <menuitem action='FindAgain'/>
             <menuitem action='FindBack'/>
-        </menu>
-
-        <menu action='ReplaceMenu'>
+            <separator/>
             <menuitem action='ReplaceInNode'/>
             <menuitem action='ReplaceInNodes'/>
             <menuitem action='ReplaceInSelNSub'/>
@@ -414,7 +418,6 @@ UI_INFO = """
         </menu>
 
         <menu action='BookmarksMenu'>
-            <menuitem action='BookmarkThisNode'/>
             <menuitem action='BookmarksHandle'/>
         </menu>
 
@@ -508,7 +511,6 @@ def get_entries(inst):
     ( "TreeExportMenu", "export_from_cherrytree", _("Nodes E_xport") ),
     ( "ChangeCaseMenu", "case_toggle", _("C_hange Case") ),
     ( "SearchMenu", None, _("_Search") ),
-    ( "ReplaceMenu", None, _("_Replace") ),
     ( "ViewMenu", None, _("_View") ),
     ( "BookmarksMenu", None, _("_Bookmarks") ),
     ( "ImportMenu", None, _("_Import") ),
@@ -541,9 +543,11 @@ def get_entries(inst):
     ( "UpCase", "case_upper", _("_Upper Case of Selection/Word"), "<control><shift>W", _("Upper the Case of the Selection/the Underlying Word"), inst.text_selection_upper_case),
     ( "ToggleCase", "case_toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
     ( "EnaDisSpellCheck", "gtk-spell-check", _("Enable/Disable _Spell Check"), "<control><alt>S", _("Toggle Enable/Disable Spell Check"), inst.toggle_ena_dis_spellcheck),
+    ( "CutPlainText", "edit-cut", _("Cu_t as Plain Text"), "<control><shift>X", _("Cut as Plain Text, Discard the Rich Text Formatting"), inst.cut_as_plain_text),
+    ( "CopyPlainText", "edit-copy", _("_Copy as Plain Text"), "<control><shift>C", _("Copy as Plain Text, Discard the Rich Text Formatting"), inst.copy_as_plain_text),
     ( "PastePlainText", "edit-paste", _("_Paste as Plain Text"), "<control><shift>V", _("Paste as Plain Text, Discard the Rich Text Formatting"), inst.paste_as_plain_text),
-    ( "CutRow", "edit-cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
-    ( "CopyRow", "edit-copy", _("_Copy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
+    ( "CutRow", "edit-cut", _("Cu_t Row"), "<shift><alt>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ( "CopyRow", "edit-copy", _("_Copy Row"), "<shift><alt>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
     ( "DeleteRow", "edit-delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
     ( "DuplicateRow", "gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
     ( "MoveRowUp", "gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
@@ -569,8 +573,10 @@ def get_entries(inst):
     ( "JustifyLeft", "gtk-justify-left", _("Justify _Left"), None, _("Justify Left the Current Paragraph"), inst.apply_tag_justify_left),
     ( "JustifyCenter", "gtk-justify-center", _("Justify _Center"), None, _("Justify Center the Current Paragraph"), inst.apply_tag_justify_center),
     ( "JustifyRight", "gtk-justify-right", _("Justify _Right"), None, _("Justify Right the Current Paragraph"), inst.apply_tag_justify_right),
+    ( "JustifyFill", "gtk-justify-fill", _("Justify _Fill"), None, _("Justify Fill the Current Paragraph"), inst.apply_tag_justify_fill),
     ( "TreeAddNode", "tree-node-add", _("Add _Node"), "<control>N", _("Add a Node having the same Father of the Selected Node"), inst.node_add),
     ( "TreeAddSubNode", "tree-subnode-add", _("Add _SubNode"), "<control><shift>N", _("Add a Child Node to the Selected Node"), inst.node_child_add),
+    ( "TreeDuplicateNode", "tree-node-dupl", _("_Duplicate Node"), "<control><shift>D", _("Duplicate the Selected Node"), inst.node_duplicate),
     ( "NodeEdit", "cherry_edit", _("Change Node _Properties"), "F2", _("Edit the Properties of the Selected Node"), inst.node_edit),
     ( "NodeToggleRO", "cherry_edit", _("Toggle _Read Only"), "<Ctrl><Alt>R", _("Toggle the Read Only Property of the Selected Node"), inst.node_toggle_read_only),
     ( "NodeDate", "calendar", _("Insert Today's Node"), "F8", _("Insert a Node with Hierarchy Year/Month/Day"), inst.node_date),
@@ -609,7 +615,6 @@ def get_entries(inst):
     ( "IncreaseToolbarIconsSize", "gtk-add", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
     ( "DecreaseToolbarIconsSize", "gtk-remove", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
     ( "Fullscreen", "gtk-fullscreen", _("_Full Screen On/Off"), "F11", _("Toggle Full Screen On/Off"), inst.fullscreen_toggle),
-    ( "BookmarkThisNode", "gtk-add", _("_Bookmark This Node"), None, _("Add the Current Node to the Bookmarks List"), inst.bookmark_curr_node),
     ( "BookmarksHandle", "gtk-edit", _("_Handle Bookmarks"), None, _("Handle the Bookmarks List"), inst.bookmarks_handle),
     ( "FromCherryTree", "from_cherrytree", _("From _CherryTree File"), None, _("Add Nodes of a CherryTree File to the Current Tree"), inst.nodes_add_from_cherrytree_file),
     ( "FromTxtFile", "from_txt", _("From _Plain Text File"), None, _("Add Node from a Plain Text File to the Current Tree"), inst.nodes_add_from_plain_text_file),
@@ -691,10 +696,12 @@ def get_popup_menu_tree(inst):
     return [
     ("tree-node-add", _("Add _Node"), "<control>N", _("Add a Node having the same Father of the Selected Node"), inst.node_add),
     ("tree-subnode-add", _("Add _SubNode"), "<control><shift>N", _("Add a Child Node to the Selected Node"), inst.node_child_add),
+    ("tree-node-dupl", _("_Duplicate Node"), "<control><shift>D", _("Duplicate the Selected Node"), inst.node_duplicate),
     (TAG_SEPARATOR, None, None, None, None),
     ("cherry_edit", _("Change Node _Properties"), "F2", _("Edit the Properties of the Selected Node"), inst.node_edit),
     ("cherry_edit", _("Toggle _Read Only"), "<Ctrl><Alt>R", _("Toggle the Read Only Property of the Selected Node"), inst.node_toggle_read_only),
-    ("gtk-add", _("_Bookmark This Node"), None, _("Add the Current Node to the Bookmarks List"), inst.bookmark_curr_node),
+    ("pin-add", _("Add to _Bookmarks"), "<control><shift>B", _("Add the Current Node to the Bookmarks List"), inst.bookmark_curr_node),
+    ("pin-remove", _("_Remove from Bookmarks"), "<control><alt>B", _("Remove the Current Node from the Bookmarks List"), inst.bookmark_curr_node_remove),
     ("calendar", _("Insert Today's Node"), "F8", _("Insert a Node with Hierarchy Year/Month/Day"), inst.node_date),
     ("gtk-info", _("Tree _Info"), None, _("Tree Summary Information"), inst.tree_info),
     (TAG_SEPARATOR, None, None, None, None),
@@ -759,6 +766,8 @@ def get_popup_menu_entries_text(inst):
     # "submenu-end", None, None, None, None
     return [
     (TAG_SEPARATOR, None, None, None, None),
+    ("edit-cut", _("Cu_t as Plain Text"), "<control><shift>X", _("Cut as Plain Text, Discard the Rich Text Formatting"), inst.cut_as_plain_text),
+    ("edit-copy", _("_Copy as Plain Text"), "<control><shift>C", _("Copy as Plain Text, Discard the Rich Text Formatting"), inst.copy_as_plain_text),
     ("edit-paste", _("_Paste as Plain Text"), "<control><shift>V", _("Paste as Plain Text, Discard the Rich Text Formatting"), inst.paste_as_plain_text),
     (TAG_SEPARATOR, None, None, None, None),
     ("submenu-start", _("For_matting") , "format_text", None, None),
@@ -783,6 +792,7 @@ def get_popup_menu_entries_text(inst):
     ("gtk-justify-left", _("Justify _Left"), None, _("Justify Left the Current Paragraph"), inst.apply_tag_justify_left),
     ("gtk-justify-center", _("Justify _Center"), None, _("Justify Center the Current Paragraph"), inst.apply_tag_justify_center),
     ("gtk-justify-right", _("Justify _Right"), None, _("Justify Right the Current Paragraph"), inst.apply_tag_justify_right),
+    ("gtk-justify-fill", _("Justify _Fill"), None, _("Justify Fill the Current Paragraph"), inst.apply_tag_justify_fill),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_List") , "list_bulleted", None, None),
     ("list_bulleted", _("Set/Unset _Bulleted List"), None, _("Set/Unset the Current Paragraph/Selection as a Bulleted List"), inst.list_bulleted_handler),
@@ -807,8 +817,8 @@ def get_popup_menu_entries_text(inst):
     ("case_toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_Row") , "gtk-edit", None, None),
-    ("edit-cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
-    ("edit-copy", _("C_opy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
+    ("edit-cut", _("Cu_t Row"), "<shift><alt>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ("edit-copy", _("C_opy Row"), "<shift><alt>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
     ("edit-delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
     ("gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
     ("gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
@@ -849,8 +859,8 @@ def get_popup_menu_entries_code(inst):
     ("case_toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_Row") , "gtk-edit", None, None),
-    ("edit-cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
-    ("edit-copy", _("C_opy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
+    ("edit-cut", _("Cu_t Row"), "<shift><alt>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ("edit-copy", _("C_opy Row"), "<shift><alt>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
     ("edit-delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
     ("gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
     ("gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
