@@ -20,7 +20,7 @@
 #       MA 02110-1301, USA.
 
 import gtk, gobject
-import re, cgi
+import re, cgi, time
 import cons, support, config
 
 
@@ -38,6 +38,7 @@ class FindReplace:
         self.newline_trick = False
         self.allmatchesdialog_init()
         self.iteratedfinddialog = None
+        self.latest_node_offset = {}
 
     def iterated_find_dialog_exist_or_create(self):
         """Exist or Create the Iterated Find Dialog"""
@@ -99,6 +100,7 @@ class FindReplace:
         """Search for a pattern in the selected Node"""
         entry_hint = ""
         if not self.from_find_iterated:
+            self.latest_node_offset = {}
             iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
             iter_bound = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_selection_bound())
             entry_predefined_text = self.dad.curr_buffer.get_text(iter_insert, iter_bound)
@@ -142,9 +144,16 @@ class FindReplace:
             self.iterated_find_dialog()
         if user_active_restore: self.dad.user_active = True
 
+    def update_all_matches_progress(self):
+        self.dad.progressbar.set_fraction(float(self.processed_nodes)/self.dad.num_nodes)
+        if self.matches_num != self.latest_matches:
+            self.latest_matches = self.matches_num
+            self.dad.progressbar.set_text(str(self.matches_num))
+
     def find_in_all_nodes(self, father_tree_iter):
         """Search for a pattern in all the Tree Nodes"""
         if not self.from_find_iterated:
+            self.latest_node_offset = {}
             iter_insert = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_insert())
             iter_bound = self.dad.curr_buffer.get_iter_at_mark(self.dad.curr_buffer.get_selection_bound())
             entry_predefined_text = self.dad.curr_buffer.get_text(iter_insert, iter_bound)
@@ -188,11 +197,20 @@ class FindReplace:
             self.dad.user_active = False
             user_active_restore = True
         else: user_active_restore = False
+        self.processed_nodes = 0
+        self.latest_matches = 0
+        self.dad.update_num_nodes()
+        if all_matches:
+            self.dad.progressbar.set_text("0")
+            self.dad.progresstop.show()
+            self.dad.progressbar.show()
+            while gtk.events_pending(): gtk.main_iteration()
+        search_start_time = time.time()
         while node_iter:
             self.all_matches_first_in_node = True
             while self.parse_given_node_content(node_iter, pattern, forward, first_fromsel, all_matches):
                 self.matches_num += 1
-                if not all_matches: break
+                if not all_matches or self.dad.progress_stop: break
             if self.matches_num == 1 and not all_matches: break
             if father_tree_iter and not self.from_find_iterated: break
             last_top_node_iter = node_iter.copy() # we need this if we start from a node that is not in top level
@@ -208,6 +226,12 @@ class FindReplace:
                     if forward: node_iter = self.dad.treestore.iter_next(node_iter)
                     else: node_iter = self.dad.get_tree_iter_prev_sibling(self.dad.treestore, node_iter)
                 else: break
+            if self.dad.progress_stop: break
+            self.processed_nodes += 1
+            if all_matches:
+                self.update_all_matches_progress()
+        search_end_time = time.time()
+        print search_end_time - search_start_time, "sec"
         if user_active_restore: self.dad.user_active = True
         config.set_tree_expanded_collapsed_string(self.dad)
         if not self.matches_num or all_matches:
@@ -227,6 +251,11 @@ class FindReplace:
                 self.dad.treeview_safe_set_cursor(self.dad.curr_tree_iter)
                 if self.dad.search_replace_dict['idialog']:
                     self.iterated_find_dialog()
+        if all_matches:
+            assert self.processed_nodes == self.dad.num_nodes or self.dad.progress_stop
+            self.dad.progresstop.hide()
+            self.dad.progressbar.hide()
+            self.dad.progress_stop = False
 
     def find_a_node(self):
         """Search for a pattern between all the Node's Names"""
@@ -309,7 +338,7 @@ class FindReplace:
             pattern = re.compile(pattern, re.UNICODE|re.MULTILINE)
         else: pattern = re.compile(pattern, re.IGNORECASE|re.UNICODE|re.MULTILINE)
         start_offset = start_iter.get_offset()
-        start_offset -= self.get_num_objs_before_offset(text_buffer, start_offset)
+        #start_offset -= self.get_num_objs_before_offset(text_buffer, start_offset)
         if forward:
             match = pattern.search(text, start_offset)
         else:
@@ -360,7 +389,7 @@ class FindReplace:
             text_buffer.insert_at_cursor(replacer_text)
             if not all_matches:
                 self.dad.set_selection_at_offset_n_delta(match_offsets[0] + num_objs, len(replacer_text))
-            self.dad.state_machine.update_state(self.dad.treestore[tree_iter][3])
+            self.dad.state_machine.update_state()
             self.dad.ctdb_handler.pending_edit_db_node_buff(self.dad.treestore[tree_iter][3], force_user_active=True)
         return True
     
@@ -405,34 +434,48 @@ class FindReplace:
         num_objs = 0
         local_limit_offset = max_offset
         curr_iter = text_buffer.get_start_iter()
-        while curr_iter.get_offset() <= local_limit_offset:
+        curr_offset = curr_iter.get_offset()
+        while curr_offset <= local_limit_offset:
             anchor = curr_iter.get_child_anchor()
             if anchor:
                 num_objs += 1
                 local_limit_offset += 1
-            if not curr_iter.forward_char(): break
+            if not curr_iter.forward_char():
+                break
+            next_offset = curr_iter.get_offset()
+            if next_offset == curr_offset:
+                break
+            curr_offset = next_offset
         return num_objs
 
-    def get_inner_start_iter(self, text_buffer, forward):
+    def get_inner_start_iter(self, text_buffer, forward, node_id):
         """Get start_iter when not at beginning or end"""
-        iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-        iter_bound = text_buffer.get_iter_at_mark(text_buffer.get_selection_bound())
+        if text_buffer.get_has_selection():
+            iter_start, iter_end = text_buffer.get_selection_bounds()
+            offsets = [iter_start.get_offset(), iter_end.get_offset()]
+        else:
+            iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+            offsets = [iter_insert.get_offset()]
         if not self.replace_active or self.replace_subsequent:
             # it's a find or subsequent replace, so we want, given a selected word, to find for the subsequent one
             if forward:
-                if iter_bound != None and iter_insert.compare(iter_bound) < 0: start_iter = iter_bound
-                else: start_iter = iter_insert
+                start_iter = text_buffer.get_iter_at_offset(max(offsets))
             else:
-                if iter_bound != None and iter_insert.compare(iter_bound) > 0: start_iter = iter_bound
-                else: start_iter = iter_insert
+                start_iter = text_buffer.get_iter_at_offset(min(offsets))
         else:
             # it's a first replace, so we want, given a selected word, to replace starting from this one
             if forward:
-                if iter_bound != None and iter_insert.compare(iter_bound) > 0: start_iter = iter_bound
-                else: start_iter = iter_insert
+                start_iter = text_buffer.get_iter_at_offset(min(offsets))
             else:
-                if iter_bound != None and iter_insert.compare(iter_bound) < 0: start_iter = iter_bound
-                else: start_iter = iter_insert
+                start_iter = text_buffer.get_iter_at_offset(max(offsets))
+        if self.latest_node_offset\
+        and self.latest_node_offset["n"] == node_id\
+        and self.latest_node_offset["o"] == start_iter.get_offset():
+            if forward: start_iter.forward_char()
+            else: start_iter.backward_char()
+        self.latest_node_offset["n"] = node_id
+        self.latest_node_offset["o"] = start_iter.get_offset()
+        #print self.latest_node_offset["n"], offsets, self.latest_node_offset["o"]
         return start_iter
 
     def parse_node_content_iter(self, tree_iter, text_buffer, pattern, forward, first_fromsel, all_matches, first_node):
@@ -448,7 +491,8 @@ class FindReplace:
             restore_modified = False
         if (first_fromsel and first_node)\
         or (all_matches and not self.all_matches_first_in_node):
-            start_iter = self.get_inner_start_iter(text_buffer, forward)
+            node_id = self.dad.get_node_id_from_tree_iter(tree_iter)
+            start_iter = self.get_inner_start_iter(text_buffer, forward, node_id)
         else:
             if forward: start_iter = text_buffer.get_start_iter()
             else: start_iter = text_buffer.get_end_iter()
@@ -478,15 +522,18 @@ class FindReplace:
             if self.parse_node_content_iter(node_iter, text_buffer, pattern, forward, first_fromsel, all_matches, False):
                 return True # not first_node node
         node_iter = self.dad.treestore.iter_children(node_iter) # check for children
-        if node_iter != None and not forward: node_iter = self.dad.get_tree_iter_last_sibling(node_iter)
-        while node_iter != None:
+        if node_iter and not forward: node_iter = self.dad.get_tree_iter_last_sibling(node_iter)
+        while node_iter and not self.dad.progress_stop:
             self.all_matches_first_in_node = True
             while self.parse_given_node_content(node_iter, pattern, forward, first_fromsel, all_matches):
                 self.matches_num += 1
-                if not all_matches: break
+                if not all_matches or self.dad.progress_stop: break
             if self.matches_num == 1 and not all_matches: break
             if forward: node_iter = self.dad.treestore.iter_next(node_iter)
             else: node_iter = self.dad.get_tree_iter_prev_sibling(self.dad.treestore, node_iter)
+            self.processed_nodes += 1
+            if all_matches:
+                self.update_all_matches_progress()
         return False
 
     def parse_node_name(self, node_iter, pattern, forward, all_matches):
